#include "e.h"
#include "e_mod_main.h"

typedef struct _Colors_CFData C_CFData;
#define IFDUP(src, dst) if(src) dst = strdup(src); else dst = NULL

struct _Colors_CFData
{
   /*- BASIC -*/
   char *redPtr;
   char *bluePtr;
   char *greenPtr;
   char *alphaPtr;
   /*- ADVANCED -*/
   char *redoutlinePtr;
   char *blueoutlinePtr;
   char *greenoutlinePtr;
   char *alphaoutlinePtr;
   char *redshadowPtr;
   char *blueshadowPtr;
   char *greenshadowPtr;
   char *alphashadowPtr;
   /*- common -*/
   c_array *ColorsToModify_L;
};

/* local subsystem functions */

static void	      _color_edit_free_data(E_Config_Dialog *cfd, void *data);
static void           color_edit_fill_data(C_CFData *c_cfdata);
static void          *_color_edit_create_data(E_Config_Dialog *cfd);
static void           _color_edit_free_data(E_Config_Dialog *cfd, void *data);
static int            _color_edit_basic_apply_data(E_Config_Dialog *cfd, void *data);
static int            _color_edit_advanced_apply_data(E_Config_Dialog *cfd, void *data);
static Evas_Object   *_color_edit_basic_create_widgets(E_Config_Dialog *cfd,Evas *evas,void *data);
static Evas_Object   *_color_edit_advanced_create_widgets(E_Config_Dialog *cfd,Evas *evas,void *data);
    
/* externally accessible functions */
/***************************************************
/ Function: 
/ Purpose:  create dialog for editng colorclasses
/
/*****************************************************/
void
color_edit_show(void *con,void *ColorsToModify)
{
   E_Config_Dialog *cfd;
   E_Config_Dialog_View v;   

   /* methods */
   v.create_cfdata           = _color_edit_create_data;
   v.free_cfdata             = _color_edit_free_data;
   v.basic.apply_cfdata      = _color_edit_basic_apply_data;
   v.basic.create_widgets    = _color_edit_basic_create_widgets;
   v.advanced.apply_cfdata   = _color_edit_advanced_apply_data;
   v.advanced.create_widgets = _color_edit_advanced_create_widgets;
   /* create config diaolg */
   cfd = e_config_dialog_new(con, _("Color Editor"), NULL, 0, &v, ColorsToModify);
}

/* local subsystem functions */
/***************************************************
/ Function: 
/ Purpose:  Take data (present) and fill it into the 
/           dialog field variables
/
/*****************************************************/
static void 
color_edit_fill_data(C_CFData *c_cfdata)
{
   char temp[50];
   /*- BASIC -*/
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->red);
   IFDUP(temp,c_cfdata->redPtr);
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->blue);
   IFDUP(temp,c_cfdata->bluePtr);
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->green);
   IFDUP(temp,c_cfdata->greenPtr);
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->alpha);
   IFDUP(temp,c_cfdata->alphaPtr);
   /*- ADVANCED -*/
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->red_o);
   IFDUP(temp,c_cfdata->redoutlinePtr);
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->blue_o);
   IFDUP(temp,c_cfdata->blueoutlinePtr);
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->green_o);
   IFDUP(temp,c_cfdata->greenoutlinePtr);
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->alpha_o);
   IFDUP(temp,c_cfdata->alphaoutlinePtr);
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->red_s);
   IFDUP(temp,c_cfdata->redshadowPtr);
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->blue_s);
   IFDUP(temp,c_cfdata->blueshadowPtr);
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->green_s);
   IFDUP(temp,c_cfdata->greenshadowPtr);
   sprintf(temp,"%d",c_cfdata->ColorsToModify_L->alpha_s);
   IFDUP(temp,c_cfdata->alphashadowPtr);
}
/***************************************************
/ Function: 
/ Purpose: create an object that hold data from dialog 
/
/
/*****************************************************/
static void *
_color_edit_create_data(E_Config_Dialog *cfd)
{
   C_CFData *c_cfdata;
   int i = 0;
   char *test;

   c_cfdata = E_NEW(C_CFData, 1);
   if (!c_cfdata) return NULL;
 
   c_cfdata->ColorsToModify_L = cfd->data;

   color_edit_fill_data(c_cfdata); 

   return c_cfdata;
}
/***************************************************
/ Function: 
/ Purpose:  Free data before closing/destroying dialog
/
/
/*****************************************************/
static void
_color_edit_free_data(E_Config_Dialog *cfd, void *data)
{    
   //free(cfd->data);
   free(data);
}
/***************************************************
/ Function: 
/ Purpose:  Move data from dialog to config object 
/           after "basic" apply button presesed.
/
/*****************************************************/
static int
_color_edit_basic_apply_data(E_Config_Dialog *cfd, void *data)
{
   C_CFData *c_cfdata;
   int AllGood;
   c_cfdata = data;

//copy stuff back to ColorsToModify_L 
  /**basic**/
   AllGood = sscanf(c_cfdata->redPtr,"%d",&(c_cfdata->ColorsToModify_L->red));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->bluePtr,"%d",&(c_cfdata->ColorsToModify_L->blue));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->greenPtr,"%d",&(c_cfdata->ColorsToModify_L->green));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->alphaPtr,"%d",&(c_cfdata->ColorsToModify_L->alpha));
   if (AllGood == 0) {return -1;}

   e_config_save_queue();
   redraw_calendar(c_cfdata->ColorsToModify_L->calendar,0);

    return 1;
}
/***************************************************
/ Function: 
/ Purpose:  Move data from dialog to config object 
/           after "advanced" apply button presesed.
/
/*****************************************************/
static int
_color_edit_advanced_apply_data(E_Config_Dialog *cfd, void *data)
{
   int AllGood;
   C_CFData *c_cfdata;
//copy stuff back to ColorsToModify_L 
   c_cfdata = data;
  /**basic**/
   AllGood = sscanf(c_cfdata->redPtr,"%d",&(c_cfdata->ColorsToModify_L->red));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->bluePtr,"%d",&(c_cfdata->ColorsToModify_L->blue));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->greenPtr,"%d",&(c_cfdata->ColorsToModify_L->green));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->alphaPtr,"%d",&(c_cfdata->ColorsToModify_L->alpha));
   if (AllGood == 0) {return -1;}
  /*- ADVANCED -*/
   AllGood = sscanf(c_cfdata->redoutlinePtr,"%d",&(c_cfdata->ColorsToModify_L->red_o));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->blueoutlinePtr,"%d",&(c_cfdata->ColorsToModify_L->blue_o));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->greenoutlinePtr,"%d",&(c_cfdata->ColorsToModify_L->green_o));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->alphaoutlinePtr,"%d",&(c_cfdata->ColorsToModify_L->alpha_o));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->redshadowPtr,"%d",&(c_cfdata->ColorsToModify_L->red_s));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->blueshadowPtr,"%d",&(c_cfdata->ColorsToModify_L->blue_s));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->greenshadowPtr,"%d",&(c_cfdata->ColorsToModify_L->green_s));
   if (AllGood == 0) {return -1;}
   AllGood = sscanf(c_cfdata->alphashadowPtr,"%d",&(c_cfdata->ColorsToModify_L->alpha_s));
   if (AllGood == 0) {return -1;}

   return 1;
}

/***************************************************
/ Function: 
/ Purpose:  create widgets for basic dialog
/
/
/*****************************************************/
static Evas_Object *
_color_edit_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, void *data)
{
   C_CFData *c_cfdata;
   Evas_Object *ol, *o;
   c_cfdata = data;

printf("redptr is = %s",c_cfdata->redPtr);

   ol = e_widget_list_add(evas, 0, 0);
   o = e_widget_table_add(evas, 0);
//label on top
   e_widget_table_object_append(o, e_widget_label_add(evas, "Red"),
                                     1, 0, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_label_add(evas, "Green"),
                                     2, 0, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_label_add(evas, "Blue"),
                                     3, 0, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_label_add(evas, "Alpha"),
                                     4, 0, 1, 1,
                                     1, 1, 1, 1);
//color row
   e_widget_table_object_append(o, e_widget_label_add(evas, "main color"),
                                     0, 1, 1, 1,
                                     1, 1, 1, 1);
   {
        Evas_Object *entry;
        entry = e_widget_entry_add(evas, &(c_cfdata->redPtr));
        e_widget_min_size_set(entry, 30, 1);
        e_widget_table_object_append(o, entry,
                                     1, 1, 1, 1,
                                     1, 1, 1, 1);
    } 
    {
    	Evas_Object *entry2;
        entry2 = e_widget_entry_add(evas, &(c_cfdata->greenPtr));
        e_widget_min_size_set(entry2, 30, 1);
	e_widget_table_object_append(o, entry2,
                                     2, 1, 1, 1,
                                     1, 1, 1, 1);
    }
    {
    	Evas_Object *entry3;
        entry3 = e_widget_entry_add(evas, &(c_cfdata->bluePtr));
        e_widget_min_size_set(entry3, 30, 1);
	e_widget_table_object_append(o, entry3,
                                     3, 1, 1, 1,
                                     1, 1, 1, 1);
    }
    {
    	Evas_Object *entry4;
        entry4 = e_widget_entry_add(evas, &(c_cfdata->alphaPtr));
        e_widget_min_size_set(entry4, 30, 1);
	e_widget_table_object_append(o, entry4,
                                     4, 1, 1, 1,
                                     1, 1, 1, 1);
    }
   e_widget_list_object_append(ol, o, 1, 1, 0.5);
   return ol;
}
/***************************************************
/ Function: 
/ Purpose:  create widgets for advanced dialog
/
/
/*****************************************************/
static Evas_Object *
_color_edit_advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, void *data)
{
   Evas_Object *o, *ol;
   C_CFData *c_cfdata;
   c_cfdata = data;
   ol = e_widget_list_add(evas, 0, 0);
   o = e_widget_table_add(evas, 0);
//label on top
   e_widget_table_object_append(o, e_widget_label_add(evas, "Red"),
                                     1, 0, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_label_add(evas, "Green"),
                                     2, 0, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_label_add(evas, "Blue"),
                                     3, 0, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_label_add(evas, "Alpha"),
                                     4, 0, 1, 1,
                                     1, 1, 1, 1);

//First Row
   e_widget_table_object_append(o, e_widget_label_add(evas, "main color"),
                                     0, 1, 1, 1,
                                     1, 1, 1, 1);
   {
        Evas_Object *entry;
        entry = e_widget_entry_add(evas, &(c_cfdata->redPtr));
        e_widget_min_size_set(entry, 30, 1);
        e_widget_table_object_append(o, entry,
                                     1, 1, 1, 1,
                                     1, 1, 1, 1);
    } 
    {
    	Evas_Object *entry2;
        entry2 = e_widget_entry_add(evas, &(c_cfdata->greenPtr));
        e_widget_min_size_set(entry2, 30, 1);
	e_widget_table_object_append(o, entry2,
                                     2, 1, 1, 1,
                                     1, 1, 1, 1);
    }
    {
    	Evas_Object *entry3;
        entry3 = e_widget_entry_add(evas, &(c_cfdata->bluePtr));
        e_widget_min_size_set(entry3, 30, 1);
	e_widget_table_object_append(o, entry3,
                                     3, 1, 1, 1,
                                     1, 1, 1, 1);
    }
    {
    	Evas_Object *entry4;
        entry4 = e_widget_entry_add(evas, &(c_cfdata->alphaPtr));
        e_widget_min_size_set(entry4, 30, 1);
	e_widget_table_object_append(o, entry4,
                                     4, 1, 1, 1,
                                     1, 1, 1, 1);
    }
//Second Row
    e_widget_table_object_append(o, e_widget_label_add(evas, "--outline"),
                                     0, 2, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_entry_add(evas, &(c_cfdata->redoutlinePtr)),
                                     1, 2, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_entry_add(evas, &(c_cfdata->greenoutlinePtr)),
                                     2, 2, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_entry_add(evas, &(c_cfdata->blueoutlinePtr)),
                                     3, 2, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_entry_add(evas, &(c_cfdata->alphaoutlinePtr)),
                                     4, 2, 1, 1,
                                     1, 1, 1, 1);
// third row
   e_widget_table_object_append(o, e_widget_label_add(evas, "--shadow"),
                                     0, 3, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_entry_add(evas, &(c_cfdata->redshadowPtr)),
                                     1, 3, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_entry_add(evas, &(c_cfdata->greenshadowPtr)),
                                     2, 3, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_entry_add(evas, &(c_cfdata->blueshadowPtr)),
                                     3, 3, 1, 1,
                                     1, 1, 1, 1);
   e_widget_table_object_append(o, e_widget_entry_add(evas, &(c_cfdata->alphashadowPtr)),
                                     4, 3, 1, 1,
                                     1, 1, 1, 1);
   e_widget_list_object_append(ol, o, 1, 1, 0.5);
   return ol;
}
